/*
 * EventConverters.java
 *
 * Created on August 26, 2002, 1:16 PM
 */

package edu.cmu.cs.able.gaugeInfrastructure.Siena.Events;
import edu.cmu.cs.able.gaugeInfrastructure.util.*;
import edu.cmu.cs.able.gaugeInfrastructure.Events.*;
import edu.cmu.cs.able.gaugeInfrastructure.Siena.SienaEventBus;
import siena.*;
/**
 *
 * @author  schmerl
 */
public class EventConverters {
    
    public static Notification pack (ConfiguredEvent event) {
        Notification n = new Notification ();
        n.putAttribute (SienaEventBus.EVENTTYPE, SienaEventBus.CONFIGURED);
        SienaEventBus.addGaugeInfo (event, n);
        n.putAttribute (SienaEventBus.STATUS, event.status?"Ok":"Err");
        SienaEventBus.SPV2Notification (event.configParams, n, SienaEventBus.CONFIG);
        return n;        
    }
    
    public static void unpack (Notification n, ConfiguredEvent event) {
        SienaEventBus.extractGaugeID (event, n);
        event.status = n.getAttribute (SienaEventBus.STATUS).stringValue ().equals ("Ok");
        if (event.status) {
            event.configParams = new StringPairVector ();
            SienaEventBus.Notification2SPV (n, event.configParams, SienaEventBus.CONFIG);
        }   
    }
    
    public static Notification pack (CreatedEvent event) {
        Notification n = new Notification ();
        n.putAttribute (SienaEventBus.EVENTTYPE, SienaEventBus.CREATED);
        SienaEventBus.addGaugeInfo (event, n);
        n.putAttribute (SienaEventBus.STATUS, event.status?"Ok":"Err");
        return n;        
    }
    
    public static void unpack (Notification n, CreatedEvent event) {
        SienaEventBus.extractGaugeID (event, n);
        event.status = n.getAttribute (SienaEventBus.STATUS).stringValue ().equals ("Ok");
    }
    
    public static Notification pack (DeletedEvent event) {
        Notification n = new Notification ();
        n.putAttribute (SienaEventBus.EVENTTYPE, SienaEventBus.DELETED);
        SienaEventBus.addGaugeInfo (event, n);
        n.putAttribute (SienaEventBus.GAUGEMGRID, event.gaugeMgrID);
        n.putAttribute (SienaEventBus.STATUS, event.status?"Ok":"Err");
        return n;        
    }
    
    public static void unpack (Notification n, DeletedEvent event) {
        SienaEventBus.extractGaugeID (event, n);
        event.gaugeMgrID = n.getAttribute (SienaEventBus.GAUGEMGRID).stringValue ();
        event.status = n.getAttribute (SienaEventBus.STATUS).stringValue ().equals ("Ok");     
    }
    
    public static Notification pack (MultipleValueEvent event) {
        Notification n = new Notification ();
        n.putAttribute (SienaEventBus.EVENTTYPE, SienaEventBus.MULTIPLEVALUES);
        SienaEventBus.addGaugeInfo (event, n);
        SienaEventBus.GVV2Notification (event.values, n, SienaEventBus.VALUE);
        return n;        
    }
    
    public static void unpack (Notification n, MultipleValueEvent event) {
        SienaEventBus.extractGaugeID (event, n);
        event.values = new GaugeValueVector ();
        SienaEventBus.Notification2GVV (n, event.values, SienaEventBus.VALUE);
    }
    
    public static Notification pack (ValueEvent event) {
        Notification n = new Notification ();
        n.putAttribute (SienaEventBus.EVENTTYPE, SienaEventBus.VALUE);
        SienaEventBus.addGaugeInfo (event, n);
        n.putAttribute (SienaEventBus.VALUENAME, event.valueName);
        n.putAttribute (SienaEventBus.PROPERTYNAME, event.propertyName);
        n.putAttribute (SienaEventBus.VALUE, event.value);
        return n;        
    }
    
    public static void unpack (Notification n, ValueEvent event) {
        SienaEventBus.extractGaugeID (event, n);
        event.valueName = n.getAttribute (SienaEventBus.VALUENAME).stringValue ();
        event.propertyName = n.getAttribute (SienaEventBus.PROPERTYNAME).stringValue ();
        event.value = n.getAttribute (SienaEventBus.VALUE).stringValue ();
    }
    
    
    public static GaugeEvent notification2Event (Notification e) {
        String eventType = e.getAttribute (SienaEventBus.EVENTTYPE).stringValue ();
        GaugeEvent returnEvent = null;
        
        if (eventType.equals (SienaEventBus.CREATED)) {
            CreatedEvent event = new CreatedEvent ();
            unpack (e, event);
            returnEvent = event;
        }
        else if (eventType.equals (SienaEventBus.CONFIGURED)) {
            ConfiguredEvent event = new ConfiguredEvent ();
            unpack (e, event);
            returnEvent = event;
        }
        else if (eventType.equals (SienaEventBus.VALUE)) {
            ValueEvent event = new ValueEvent ();
            unpack (e, event);
            returnEvent = event;
        }
        else if (eventType.equals (SienaEventBus.MULTIPLEVALUES)) {
            MultipleValueEvent event = new MultipleValueEvent ();
            unpack (e, event);
            returnEvent = event;
        }
        else if (eventType.equals (SienaEventBus.DELETED)) {
            DeletedEvent event = new DeletedEvent ();
            unpack (e, event);
            returnEvent = event;
        }
        return returnEvent;
    }
    
    public static Filter event2Filter(GaugeEvent event) {
        Filter f = new Filter();
        addConstraint(f, "gaugeID", event.uid);
        addConstraint(f, "gaugeName", event.gaugeName);
        addConstraint(f, "gaugeType", event.gaugeType);
        addConstraint(f, "modelName", event.modelName);
        addConstraint(f, "modelType", event.modelType);
        
        addEventSpecificConstraints(f, event);
        return f;
    }

    
    private static void addConstraint (Filter f, String tag, String pattern) {
        if (pattern != null && !pattern.equals ("")) {
            f.addConstraint (tag, pattern);
        }
    }
    
    private static void addEventSpecificConstraints (Filter f, GaugeEvent event) {
        if (event instanceof ConfiguredEvent) {
            ConfiguredEvent ce = (ConfiguredEvent )event;
            addConstraint (f, SienaEventBus.EVENTTYPE, SienaEventBus.CONFIGURED);
            // TODO: config params
        }
        else if (event instanceof CreatedEvent) {
            CreatedEvent ce = (CreatedEvent )event;
            addConstraint (f, SienaEventBus.EVENTTYPE, SienaEventBus.CREATED);
            // TODO: setup and mappings
        }
        else if (event instanceof DeletedEvent) {
            DeletedEvent de = (DeletedEvent )event;
            addConstraint (f, SienaEventBus.EVENTTYPE, SienaEventBus.DELETED);
        }
        else if (event instanceof MultipleValueEvent) {
            MultipleValueEvent mve = (MultipleValueEvent )event;
            addConstraint (f, SienaEventBus.EVENTTYPE, SienaEventBus.MULTIPLEVALUES);
            // TODO: add values
        }
        else if (event instanceof ValueEvent) {
            ValueEvent ve = (ValueEvent )event;
            addConstraint (f, SienaEventBus.EVENTTYPE, SienaEventBus.VALUE);
            addConstraint (f, SienaEventBus.VALUENAME, ve.valueName);
            addConstraint (f, SienaEventBus.PROPERTYNAME, ve.propertyName);
            addConstraint (f, SienaEventBus.VALUE, ve.value);
        }
    }
}