/*
 * GMgrSideEventBus.java
 *
 * Created on August 26, 2002, 2:39 PM
 */

package edu.cmu.cs.able.gaugeInfrastructure.Siena;
import siena.*;
import edu.cmu.cs.able.gaugeInfrastructure.*;
import edu.cmu.cs.able.gaugeInfrastructure.util.*;
import edu.cmu.cs.able.gaugeInfrastructure.Events.*;
import edu.cmu.cs.able.gaugeInfrastructure.Siena.*;
import edu.cmu.cs.able.gaugeInfrastructure.Siena.Events.*;
import java.util.*;
/**
 *
 * @author  schmerl
 */
public class GMgrSideEventBus extends SienaEventBus {
    
    /** Gauge managers registered to this bus
     */
    private Vector gaugeMgrs = new Vector ();
    private Vector bufferedCreationEvents = new Vector ();
    private Hashtable bufferedDeleteEvents = new Hashtable ();
    
    
    /** Creates a new instance of GMgrSideEventBus */
    public GMgrSideEventBus (String senp) {
        initializeSiena (senp);
    }
    
    public void notify (siena.Notification e) throws siena.SienaException {
        try {
        // Handles:
        // findGaugeMgrEvent (handled internally by bus)
        // GaugeInitEvents (sent by a gauge after the process has been created)
        // CreateGaugeEvent, DeleteGaugeEvent, QueryMetaInfoEvent, which are processed by the gauge mgr

        if (!gaugeMgrs.isEmpty ()) {

            String eventType = e.getAttribute (EVENTTYPE).stringValue ();
      
            if (eventType.equals (FINDGMGR)) {
                String initiator = e.getAttribute (INITIATOR).stringValue ();
                FindGaugeMgrEvent event = FindGaugeMgrEvent.unpackRequest (e);
                Enumeration mgrs = gaugeMgrs.elements ();
                boolean foundGM = false;
                while (mgrs.hasMoreElements () && !foundGM) {
                    GaugeMgr gaugeMgr = (GaugeMgr )mgrs.nextElement ();
                    if (gaugeMgr.managesType (event.gaugeType)) {
                        foundGM = true;
                        event.gaugeMgrID = gaugeMgr.getGaugeMgrID ();
                        event.status = true;
                        publishResponse (event, initiator);
                    }
                }
            }
            else if (eventType.equals (GAUGEINIT)) {
                Global.debug ("GaugeMgr processing GAUGEINIT");
                // for debugging possible gauge creation race condition (zensoul-4/30/2002)
                Global.debug (" -> SienaEventBus current thread is " + Thread.currentThread());
                // end debugging
                GaugeInitEvent init = GaugeInitEvent.unpackRequest (e);
                String initiator = e.getAttribute (INITIATOR).stringValue ();
                // find the matching creation event
                CreateGaugeEvent event = null;
                for (int i = 0; i < bufferedCreationEvents.size () && event == null; i++) {
                    CreateGaugeEvent candidate = (CreateGaugeEvent )bufferedCreationEvents.elementAt (i);
                    Global.debug ("Comparing " + candidate.toString () + " with " + init.toString ());
                    if (candidate.equals (init)) {
                        event = candidate;
                    }
                }
                // Send gauge setups and mappings
                init.setup = event.setupParams;
                init.mappings = event.mappings;
                init.status = true;
                publishResponse (init, initiator);
            }
            else if (eventType.equals (FINISHEDCREATION)) {
                CreationFinishedEvent fin = CreationFinishedEvent.unpackRequest (e);
                CreateGaugeEvent event = null;
                // bufferedCreationEvents contains all the outstanding gauge creation requests
                // when FINISHEDGAUGECREATION comes from a gauge to the gauge mgr,
                // the gauge mgr needs find the appropriate createGauge msg from the consumer
                // and reply, otherwise the consumer will time out

                for (int i = 0; i < bufferedCreationEvents.size () && event == null; i++) {
                    CreateGaugeEvent candidate = (CreateGaugeEvent )bufferedCreationEvents.elementAt (i);
                    Global.debug ("Comparing " + candidate.toString () + " with " + fin.toString ());
                    if (candidate.equals (fin)) {
                        event = candidate;
                        bufferedCreationEvents.removeElementAt (i);
                    }
                }
                event.status = fin.status;
                String consumer = event.gaugeMgrID;
                event.gaugeMgrID = fin.gaugeMgrID;
                publishResponse (event, consumer);
            }
            else if (eventType.equals (CREATE)) {
                Global.debug ("GaugeMgr processing CREATE");
                // for debugging possible gauge creation race condition (zensoul-4/30/2002)
                Global.debug(" -> SienaEventBus current thread is " + Thread.currentThread());
                // end debugging
                String initiator = e.getAttribute (INITIATOR).stringValue ();             
                CreateGaugeEvent event = CreateGaugeEvent.unpackRequest (e);
                Enumeration mgrs = gaugeMgrs.elements ();
                boolean foundGM = false;
                while (mgrs.hasMoreElements () && !foundGM) {
                    GaugeMgr gaugeMgr = (GaugeMgr )mgrs.nextElement ();

                     if (gaugeMgr.getGaugeMgrID ().equals (event.gaugeMgrID)) {
                        foundGM = true;
                        event.gaugeMgrID = initiator; // hack to remember initiator
                        bufferedCreationEvents.addElement (event);
                        GaugeControl gauge = gaugeMgr.createGauge (event, event.setupParams, event.mappings);

                        // The creation is finished in the GaugeInit event handler
                    }
                }
            }
            else if (eventType.equals (DELETE)) {
                String initiator = e.getAttribute (INITIATOR).stringValue ();
                DeleteGaugeEvent event = DeleteGaugeEvent.unpackRequest (e);
                Enumeration mgrs = gaugeMgrs.elements ();
                boolean foundGM = false;
                while (mgrs.hasMoreElements () && !foundGM) {
                    GaugeMgr gaugeMgr = (GaugeMgr )mgrs.nextElement ();
                    if (gaugeMgr.getGaugeMgrID ().equals (event.gaugeMgrID)) {
                        SienaGaugeMgrGaugeHandle gauge = new SienaGaugeMgrGaugeHandle (event);
                        
                        // Need to tell the gauge under the covers
                        GMgrDeleteGaugeEvent deleteEvent = new GMgrDeleteGaugeEvent (gauge.getGaugeID ());
                        deleteEvent.gaugeMgrID = gaugeMgr.getGaugeMgrID ();
                        GaugeID gid = gauge.getGaugeID ();    
                        String oldUID = gid.uid;
                        gid.uid = "";
                        Global.debug ("Adding delete for " + gid.toString ());
                        event.gaugeMgrID = initiator;
                        bufferedDeleteEvents.put (gid.toString (), event);
                        gid.uid = oldUID;
                        publishAnnouncement (deleteEvent);

                   }
                }
            }
            else if (eventType.equals (METAINFO)) {
                Global.debug ("Got MetaInfo");
                String initiator = e.getAttribute (INITIATOR).stringValue ();
                QueryMetaInfoEvent event = QueryMetaInfoEvent.unpackRequest (e);
                Enumeration mgrs = gaugeMgrs.elements ();
                boolean foundGM = false;
                while (mgrs.hasMoreElements () && !foundGM) {
                    Global.debug ("finding GM");
                    GaugeMgr gaugeMgr = (GaugeMgr )mgrs.nextElement ();
                    if (gaugeMgr.managesType (event.gaugeType)) {
                        Global.debug ("Found GM");
                        event.configParams = new StringPairVector ();
                        event.values = new StringPairVector ();
                        Global.debug ("Calling query for " + event.gaugeType);
                        event.gaugeMgrID = gaugeMgr.getGaugeMgrID ();
                        event.status = gaugeMgr.queryMetaInfo (event.gaugeType, event.configParams, event.values);
                        publishResponse (event, initiator);
                    }
                }
                
            }
            else if (eventType.equals (DELETED)) {
                Global.debug ("Got delete announcement");
                DeletedEvent del = new DeletedEvent ();
                EventConverters.unpack (e, del);
                Enumeration mgrs = gaugeMgrs.elements ();
                boolean foundGM = false;
                while (mgrs.hasMoreElements () && !foundGM) {
                    GaugeMgr gaugeMgr = (GaugeMgr )mgrs.nextElement ();
                    if (gaugeMgr.getGaugeMgrID ().equals (del.gaugeMgrID)) { 
                        foundGM = true;
                        SienaGaugeMgrGaugeHandle gauge = new SienaGaugeMgrGaugeHandle (del);
                        GaugeID gid = gauge.getGaugeID ();
                        gid.uid = "";
           
                        Global.debug ("Finding event for " + gid.toString ());
                        DeleteGaugeEvent event = (DeleteGaugeEvent )bufferedDeleteEvents.get (gid.toString ());
                        bufferedDeleteEvents.remove (gid.toString ());
                        if (event != null) {
                            event.status = gaugeMgr.deleteGauge (gauge);
                            String initiator = event.gaugeMgrID;
                            event.gaugeMgrID = gaugeMgr.getGaugeMgrID ();
                            publishResponse (event, initiator);
                        }
                        else {
                            Global.debug ("WARNING: Didn't find relevant gauge");}
                    }
                }
            }
            
        }
        }
        catch (Exception ex) {
            ex.printStackTrace ();
            if (ex instanceof SienaException) {
                throw (SienaException )ex;
            }
            else {
                throw new SienaException (ex.toString ());
            }
        }
    }
    
    protected Notification prepareAnnouncement (GaugeEvent event) {
        Notification not = null;
        if (event instanceof GMgrDeleteGaugeEvent) {
            not = ((GMgrDeleteGaugeEvent )event).packRequest ();
        }
        else if (event instanceof DeletedEvent) {
            not = EventConverters.pack ((DeletedEvent )event);
        }
        return not;
    }
    
    protected void prepareReplyFilter (GaugeEvent event, Filter f) {
        if (event instanceof GMgrDeleteGaugeEvent) {
            f.addConstraint (EVENTTYPE, GMGRDELETEGAUGE);
            f.addConstraint (GAUGEMGRID, Op.SS, ((GMgrDeleteGaugeEvent )event).gaugeMgrID);
            f.addConstraint (EVENTID, ((GMgrDeleteGaugeEvent )event).deleteID);          
        }
    }
    
    protected Notification prepareRequest (GaugeEvent event) {
        Notification not = null;
        if (event instanceof GMgrDeleteGaugeEvent) {
            not = ((GMgrDeleteGaugeEvent )event).packRequest ();
        }    
        return not;
    }
    
    protected Notification prepareResponse (GaugeEvent event) {
        // Respond to: CreateGauge, DeleteGauge, QueryMetaInfo
        Notification not = null;
        
        if (event instanceof QueryMetaInfoEvent) {
            not = ((QueryMetaInfoEvent )event).packReply ();
        }
        else if (event instanceof CreateGaugeEvent) {
            not = ((CreateGaugeEvent )event).packReply ();
        }
        else if (event instanceof DeleteGaugeEvent) {
            not = ((DeleteGaugeEvent )event).packReply ();
        }
        else if (event instanceof FindGaugeMgrEvent) {
            not = ((FindGaugeMgrEvent )event).packReply ();
        }
        return not;
    }
    
    protected GaugeEvent unpackReply (Notification reply, GaugeEvent event) {
        if (event instanceof GMgrDeleteGaugeEvent) {
            ((GMgrDeleteGaugeEvent )event).unpackReply (reply);
        }
        return event;
    }
    
    public void registerGaugeMgr (GaugeMgr gMgr) {
        Global.debug("Registering " + gMgr.getGaugeMgrID ());
        gaugeMgrs.addElement (gMgr);
        registerGaugeMgrSideEvents (gMgr);
    }

    private void registerGaugeMgrSideEvents (GaugeMgr gMgr) {
        Filter cgf = new Filter ();
        Filter dgf = new Filter ();
        Filter gif = new Filter ();
        Filter fmf = new Filter ();
        Filter qmf = new Filter ();
        Filter df = new Filter ();
        Filter fcf = new Filter ();
        Filter gmd = new Filter ();
        
        String gmid = "\"" + gMgr.getGaugeMgrID () + "\"";
        
        gmd.addConstraint (EVENTTYPE, DELETED);
        gmd.addConstraint (GAUGEMGRID, Op.SS, gMgr.getGaugeMgrID ());
        
        cgf.addConstraint (EVENTTYPE, CREATE);
        cgf.addConstraint (GAUGEMGRID, Op.SS, gMgr.getGaugeMgrID ());
        cgf.addConstraint (DIRECTION, REQUEST);
        
        dgf.addConstraint (EVENTTYPE, DELETE);
        dgf.addConstraint (GAUGEMGRID, Op.SS, gMgr.getGaugeMgrID ());
        dgf.addConstraint (DIRECTION, REQUEST);
         
        gif.addConstraint (EVENTTYPE, GAUGEINIT);
        gif.addConstraint (GAUGEMGRID, Op.SS, gMgr.getGaugeMgrID ());
        gif.addConstraint (DIRECTION, REQUEST);   
        
        fmf.addConstraint (EVENTTYPE, FINDGMGR);
        fmf.addConstraint (DIRECTION, REQUEST);
        
        qmf.addConstraint (EVENTTYPE, METAINFO);
        qmf.addConstraint (GAUGEMGRID, Op.SS, gMgr.getGaugeMgrID ());
        qmf.addConstraint (DIRECTION, REQUEST);   
        
        df.addConstraint (EVENTTYPE, DELETED);
        df.addConstraint (GAUGEMGRID, Op.SS, gMgr.getGaugeMgrID ());
        
        fcf.addConstraint (EVENTTYPE, FINISHEDCREATION);
        fcf.addConstraint (GAUGEMGRID, Op.SS, gMgr.getGaugeMgrID ());
        
              // Horrible hackaroo
        // Sometimes, things gMgrID gets published with quotes around it
        // So, we better listen to those too...
/*        Filter cgf2 = new Filter ();
        Filter dgf2 = new Filter ();
        Filter gif2 = new Filter ();
        Filter qmf2 = new Filter ();
        Filter df2 = new Filter ();
        Filter fcf2 = new Filter ();
        Filter gmd2 = new Filter ();
                
        gmd2.addConstraint (EVENTTYPE, DELETED);
        gmd2.addConstraint (GAUGEMGRID, gmid);
        
        cgf2.addConstraint (EVENTTYPE, CREATE);
        cgf2.addConstraint (GAUGEMGRID, gmid);
        cgf2.addConstraint (DIRECTION, REQUEST);
        
        dgf2.addConstraint (EVENTTYPE, DELETE);
        dgf2.addConstraint (GAUGEMGRID, gmid);
        dgf2.addConstraint (DIRECTION, REQUEST);
         
        gif2.addConstraint (EVENTTYPE, GAUGEINIT);
        gif2.addConstraint (GAUGEMGRID, gmid);
        gif2.addConstraint (DIRECTION, REQUEST);   
        
        qmf2.addConstraint (EVENTTYPE, METAINFO);
        qmf2.addConstraint (GAUGEMGRID, gmid);
        qmf2.addConstraint (DIRECTION, REQUEST);   
        
        df2.addConstraint (EVENTTYPE, DELETED);
        df2.addConstraint (GAUGEMGRID, gmid);
        
        fcf2.addConstraint (EVENTTYPE, FINISHEDCREATION);
        fcf2.addConstraint (GAUGEMGRID, gmid);
 **/
        
        try {
            siena.subscribe (cgf, this);
            siena.subscribe (dgf, this);
            siena.subscribe (gif, this);
            siena.subscribe (fmf, this);
            siena.subscribe (qmf, this);
            siena.subscribe (df, this);
            siena.subscribe (fcf, this);
            siena.subscribe (gmd, this);
            
/*                        siena.subscribe (cgf2, this);
            siena.subscribe (dgf2, this);
            siena.subscribe (gif2, this);
            siena.subscribe (qmf2, this);
            siena.subscribe (df2, this);
            siena.subscribe (fcf2, this);
            siena.subscribe (gmd2, this);
  */          

            
        }
        catch (SienaException ex) {
            System.err.println ("Gauge Manager: " + gMgr.getGaugeMgrID () + ": Error subscribing to Siena bus.");
            ex.printStackTrace ();
        }
    }
}
