/*
 * GaugeSideEventBus.java
 *
 * Created on August 26, 2002, 1:36 PM
 */

package edu.cmu.cs.able.gaugeInfrastructure.Siena;
import siena.*;
import edu.cmu.cs.able.gaugeInfrastructure.*;
import edu.cmu.cs.able.gaugeInfrastructure.util.*;
import edu.cmu.cs.able.gaugeInfrastructure.Events.*;
import edu.cmu.cs.able.gaugeInfrastructure.Siena.*;
import edu.cmu.cs.able.gaugeInfrastructure.Siena.Events.*;
import java.util.*;
/**
 *
 * @author  schmerl
 */
public class GaugeSideEventBus extends SienaEventBus {

    /** The list of gauges currently registered to this bus
     */
    private Vector gauges = new Vector ();
    
    /** Creates a new instance of GaugeSideEventBus */
    public GaugeSideEventBus (String senp) {
        initializeSiena (senp);
    }
    
    public void notify (siena.Notification e) throws siena.SienaException {
        try {
        GaugeID gaugeID = new GaugeID ();
        extractGaugeID (gaugeID, e);
        GaugeControl gauge = null;
        Enumeration gauges = this.gauges.elements ();
        while (gauges.hasMoreElements () && gauge == null) {
            GaugeControl candidate = (GaugeControl )gauges.nextElement ();
            if (candidate.getGaugeID ().equals (gaugeID)) {
                gauge = candidate;
            }
        }
        
        if (gauge != null) {
            String eventType = e.getAttribute (EVENTTYPE).stringValue ();
            String initiator = "";
            if (!eventType.equals (GMGRDELETEGAUGE)) initiator = e.getAttribute (INITIATOR).stringValue ();
            if (eventType.equals (CONFIGURE)) {
                ConfigureGaugeEvent reply = ConfigureGaugeEvent.unpackRequest (e); 
                
                reply.status = gauge.configure (reply.configParams);
                
                // Announce that the gauge has been configured
                ConfiguredEvent announce = new ConfiguredEvent (gaugeID);
                announce.configParams = reply.configParams;
                announce.status = reply.status;
                publishAnnouncement (announce);
                
                // Reply to sender
                publishResponse (reply, initiator);
            }
            else if (eventType.equals (QUERYSTATE)) {
                QueryGaugeStateEvent reply = QueryGaugeStateEvent.unpackRequest (e);
                reply.setupParams = new StringPairVector ();
                reply.configParams = new StringPairVector ();
                reply.mappings = new StringPairVector ();
                reply.status = gauge.queryState (reply.setupParams, reply.configParams, reply.mappings);
                publishResponse (reply, initiator);
            }
            else if (eventType.equals (QUERYVALUE)) {
                QueryValueEvent reply = QueryValueEvent.unpackRequest (e);
                reply.value = gauge.queryValue (reply.valueName);
                if (reply.value != null && !reply.value.equals ("")) reply.status = true;
                else reply.status = false;
                publishResponse (reply, initiator);
            }
            else if (eventType.equals (QUERYALLVALUES)) {
                QueryAllValuesEvent reply = QueryAllValuesEvent.unpackRequest (e);
                reply.values = new GaugeValueVector ();
                reply.status = gauge.queryAllValues (reply.values);
                publishResponse (reply, initiator);
            }
            else if (eventType.equals (GMGRDELETEGAUGE)) {
                Global.debug ("Got Deletion event ");
                GMgrDeleteGaugeEvent reply = GMgrDeleteGaugeEvent.unpackRequest (e);
                reply.status = true;
                ((SienaGauge )gauge).processDelete (reply.gaugeMgrID);
                //publishResponse (reply, initiator);
            }
        }        
        }
        catch (Exception ex) {
            ex.printStackTrace ();
            if (ex instanceof SienaException) {
                throw (SienaException )ex;
            }
            else {
                throw new SienaException (ex.toString ());
            }
        }
    }
    

    protected Notification prepareAnnouncement (GaugeEvent event) {
        Notification not = null;
        // Work out the right event, and pack it appropriately
        if (event instanceof ConfiguredEvent) {
            not = EventConverters.pack ((ConfiguredEvent )event);
        }
        else if (event instanceof CreatedEvent) {
            not = EventConverters.pack ((CreatedEvent )event);
        }
        else if (event instanceof DeletedEvent) {
            not = EventConverters.pack ((DeletedEvent )event);
        }
        else if (event instanceof MultipleValueEvent) {
            not = EventConverters.pack ((MultipleValueEvent )event);
        }
        else if (event instanceof ValueEvent) {
            not = EventConverters.pack ((ValueEvent )event);
        }
        else if (event instanceof CreationFinishedEvent) {
            not = ((CreationFinishedEvent )event).packRequest ();
        }
/*        else if (event instanceof GMgrDeleteGaugeEvent) {
            not = packGMgrDeleteGaugeRequest ((GMgrDeleteGaugeEvent )event);
        }
*/  
        return not;
    }
    
    protected void prepareReplyFilter (GaugeEvent event, Filter f) {
        // Gauge only has one call/return interface - for initialization
        if (event instanceof GaugeInitEvent) {
            GaugeInitEvent ge = (GaugeInitEvent )event;
            addGaugeInfo (ge, f);
            f.addConstraint (EVENTTYPE, GAUGEINIT);
            f.addConstraint (GAUGEMGRID, Op.SS, ge.gaugeMgrID);
        }
    }
    
    protected Notification prepareRequest (GaugeEvent event) {
        // Deal with GaugeInit
        Notification not = null;
        if (event instanceof GaugeInitEvent) {
            not = ((GaugeInitEvent )event).packRequest ();
        }
        return not;
    }
    
    protected Notification prepareResponse (GaugeEvent event) {
        // Reply to events: Configure, QueryState, QueryValue, QueryAllValues, GMgrDeleteGauge
        Notification not = null;
        
        if (event instanceof ConfigureGaugeEvent) {
           not =  ((ConfigureGaugeEvent )event).packReply ();            
        }
        else if (event instanceof QueryValueEvent) {
            not = ((QueryValueEvent )event).packReply ();
        }
        else if (event instanceof QueryAllValuesEvent) {
            not =  ((QueryAllValuesEvent )event).packReply ();
        }
        else if (event instanceof QueryGaugeStateEvent) {
            not =  ((QueryGaugeStateEvent )event).packReply ();
        }
        else if (event instanceof GMgrDeleteGaugeEvent) {
            not =  ((GMgrDeleteGaugeEvent )event).packReply ();
        }
        
        return not;
    }
    
    protected GaugeEvent unpackReply (Notification reply, GaugeEvent event) {
        if (event instanceof GaugeInitEvent) {
            ((GaugeInitEvent )event).unpackReply (reply);
        }
        return event;
    }
    
    public void registerGauge (GaugeControl gauge) {
        Global.debug ("Registering Gauge: " + gauge.getGaugeID ().toString ());
        gauges.addElement (gauge);
        registerGaugeSideEvents (gauge);
    }
    
    public void unregisterGauge (GaugeControl gauge) {
        Global.debug ("Unregistering gauge: " + gauge.getGaugeID ().toString ());
        gauges.removeElement (gauge);
        Filter f = new Filter ();
        GaugeID gaugeID = gauge.getGaugeID ();
        addGaugeInfo (gaugeID, f);
        if (gauge instanceof SienaGaugeHandle) {
            f.addConstraint (DIRECTION, ANNOUNCE);
        }
        else {
            f.addConstraint (DIRECTION, REQUEST);
        }
        siena.unsubscribe (f, this);        
    }
    
    private void registerGaugeSideEvents (GaugeControl gauge) {
        Filter f = new Filter ();
        GaugeID gaugeID = gauge.getGaugeID ();
        addGaugeInfo (gaugeID, f);
        if (gauge instanceof SienaGaugeHandle) {
            f.addConstraint (DIRECTION, ANNOUNCE);
        }
        else {
            f.addConstraint (DIRECTION, REQUEST);
        }
       

        try {
            siena.subscribe (f, this);
            Filter f2 = new Filter ();
            addGaugeInfo (gaugeID, f2);
            f2.addConstraint (EVENTTYPE, GMGRDELETEGAUGE);
            f2.addConstraint (DIRECTION, ANNOUNCE);
            if (!(gauge instanceof SienaGaugeHandle)) {
                siena.subscribe (f2, this);
            }
        }
        catch (SienaException ex) {
            System.err.println (gauge.getGaugeID ().toString () + ": Error subscribing to Siena bus.");
            ex.printStackTrace ();
        }
    }
}
