/*
 * SienaEventBus2.java
 *
 * Created on August 22, 2002, 2:03 PM
 */

package edu.cmu.cs.able.gaugeInfrastructure.Siena;
import edu.cmu.cs.able.gaugeInfrastructure.Events.*;
import edu.cmu.cs.able.gaugeInfrastructure.Siena.Events.*;
import edu.cmu.cs.able.gaugeInfrastructure.util.*;
import edu.cmu.cs.able.gaugeInfrastructure.*;
import siena.*;
import java.util.*;
/**
 *
 * @author  schmerl
 */
public abstract class SienaEventBus implements siena.Notifiable {

    class NotificationHandlerGauge {
        public Notification filterAsNotification;
        public Filter filter;
        public Object handler;
        public GaugeControl gauge;
    }
    
    /** The tag for a ConfiguredEvent.
     */    
    public final static String CONFIGURED = "gaugeConfigured";
    /** The tag for a DeletedEvent.
     */    
    public final static String DELETED = "gaugeDeleted";
    /** The tag for a CreatedEvent.
     */    
    public final static String CREATED = "gaugeCreated";
    /** The tag for a {@link MultipleValueEvent}
     */    
    public final static String MULTIPLEVALUES = "gaugeMV";
    /** The tag for a {@link ValueEvent}
     */    
    public final static String VALUE = "onReport";
    /** The tag for a {@link DeleteEvent}
     */    
    public final static String DELETE = "deleteGauge";
    /** The tag for a {@link ConfigureEvent}
     */    
    public final static String CONFIGURE = "configureGauge";
    /** The tag for a {@link CreateEvent}
     */    
    public final static String CREATE = "createGauge";
    /** The tag for a {@link QueryMetaInfoEvent}
     */    
    public final static String METAINFO = "queryGaugeMetaInfo";
    /** The tag for a {@link QueryValueEvent}
     */    
    public final static String QUERYVALUE = "queryValue";
    /** The tag for a {@link QueryAllValuesEvent}
     */    
    public final static String QUERYALLVALUES = "queryAllValues";
    /** The tag for a {@link QueryStateEvent}
     */    
    public final static String QUERYSTATE = "queryGaugeState";
    /** The tag for a {@link FindGaugeMgrEvent}
     */    
    public final static String FINDGMGR = "findGaugeMgr";
    /** The tag for event direction=request
     */    
    protected final static String REQUEST = "request";
    /** The tag for event direction=announce
     */    
    public final static String ANNOUNCE = "announce";
    /** The tag for event direction=response
     */    
    public final static String RESPONSE = "reply";
    /** The tag for {@link GaugeInitEvent}
     */    
    public final static String GAUGEINIT = "gaugeInit";
    /** The tag for {@link GMgrDeleteGaugeEvent}
     */    
    public final static String GMGRDELETEGAUGE = "GMgrDeleteGauge";
    /** The tag for a CreationFinishedEvent
     */
    public final static String FINISHEDCREATION = "FinishedCreation";
    
    /** The tag used in a Siena notification to denote the event type attribute name
     */    
    public final static String EVENTTYPE = "eventType";
    /** The tag used in a Siena notification to denote the gauge manager ID attribute name
     */    
    public final static String GAUGEMGRID = "gaugeMgr";
    /** The tag used in a Siena notification to denote the setup attribute name
     */    
    public final static String SETUP = "setup";
    /** The tag used in a Siena notification to denote the mappings attribute name
     */    
    public final static String MAPPINGS = "mapping";
    /** The tag used in a Siena notification to denote the config attribute name
     */    
    public final static String CONFIG = "config";
    /** The tag used in a Siena notification to denote the status attribute name
     */    
    public final static String STATUS = "status";
    /** The tag used in a Siena notification to denote the initiator attribute name
     */    
    public final static String INITIATOR = "initiator";
    /** The tag used in a Siena notification to denote the event ID attribute name
     */    
    public final static String EVENTID = "eventID";
    /** The tag used in a Siena notification to denote the direction attribute name
     */    
    public final static String DIRECTION = "direction";
    
    public final static String GAUGETYPE = "gaugeType";
    
    public final static String VALUENAME = "valueName";
    
    public final static String PROPERTYNAME = "propertyName";
    /** How long to wait for a blocking response (in milliseconds)
     */    
    protected final static int TIMEOUT = 10000;
    /** How many times to retry a blocking wait
     */    
    protected final static int RETRIES = 3;
    
    /** The siena bus
     */    
    public /*static*/ HierarchicalDispatcher siena = new HierarchicalDispatcher();
    
    protected String senp = null;
    /** Only one event bus can be constructed. This is it.
     */    
//    public static SienaEventBus theEventBus = null;
    
    /** The ID of this event bus - this will be used to match up event replies
     */
    protected String id = null;
    
    protected void initializeSiena (String uri) {
        try {
            senp = uri;
            siena.setMaster (senp);
        }
        catch (Exception ex) {
            ex.printStackTrace ();
        }
        id = Global.generateUniqueID ();
    }
    
    
    protected abstract Notification prepareAnnouncement (GaugeEvent event);
    
    public boolean publishAnnouncement (GaugeEvent event) {
        Notification not = null;
        not = prepareAnnouncement (event);
        
        // THis event is an announcement - nor reply expected
        not.putAttribute (DIRECTION, ANNOUNCE);
        
        // publish the event on siena
        try {
            Global.debug (not.toString ());
            siena.publish (not);
        }
        catch (SienaException ex) {
            ex.printStackTrace ();
            return false;
        }
        
        return true;
    }
     
    protected abstract Notification prepareRequest (GaugeEvent event);
    
    public GaugeEvent publishRequest (GaugeEvent event) {
        Notification not = null;
        not = prepareRequest (event);
        not.putAttribute (DIRECTION, REQUEST);
        not.putAttribute (INITIATOR, id);
        
        // Prepare for response
        Filter f = new Filter ();
        NotificationBuffer nb = prepareForReply (event, f);
        
        // publish the event using siena
        try {
            Global.debug (not.toString ());
            siena.publish (not);
        }
        catch (SienaException ex) {
            ex.printStackTrace ();
            event.status = false;
            return event;
        }
        
        GaugeEvent reply = waitForReply (event, nb, f);
        return reply;
    }
    
    protected abstract Notification prepareResponse (GaugeEvent event);
    
    public boolean publishResponse (GaugeEvent event, String initiator) {
        Notification not = null;
        not = prepareResponse (event);
        not.putAttribute (DIRECTION, RESPONSE);
        not.putAttribute (INITIATOR, initiator);
        
        try {
            Global.debug (not.toString ());
            siena.publish (not);
        }
        catch (SienaException ex) {
            ex.printStackTrace ();
            return false;
        }
        return true;
    }
    
    protected abstract void prepareReplyFilter (GaugeEvent event, Filter f);
    
    protected  NotificationBuffer prepareForReply (GaugeEvent event, Filter f) {
        prepareReplyFilter (event, f);
        f.addConstraint (INITIATOR, id);
        f.addConstraint (DIRECTION, RESPONSE);
        
        NotificationBuffer nb = new NotificationBuffer ();
        try {
            siena.subscribe (f, nb);
        }
        catch (SienaException ex) {
            ex.printStackTrace ();
        }
        return nb;
    }
    
    protected abstract GaugeEvent unpackReply (Notification reply, GaugeEvent event);
    
    private GaugeEvent waitForReply (GaugeEvent event, NotificationBuffer nb, Filter f) {
        final int TIMEOUT = 20000;
        Global.debug ("Waiting for reply: " + f.toString ());
        // Siena's blocking wait
        Notification reply = null;
        try {
            reply = nb.getNotification (TIMEOUT);           
            Global.debug (reply==null?"null":reply.toString ());       
        }
        catch (Exception ex) {
            ex.printStackTrace ();
        }
        
        if (null == reply) {
            event.status = false;
            return event;
        }
        
        return unpackReply (reply, event);
    
    }
    
    // Siena Notifiable implementation methods
    public abstract void notify (siena.Notification notification) throws siena.SienaException;    
    public void notify (siena.Notification[] notification) throws siena.SienaException {
        for (int i = 0; i < notification.length; i++) {
            notify (notification [i]);
        }
    }
       
    public static void addGaugeInfo (GaugeID gauge, Notification n) {
        n.putAttribute ("gaugeID", gauge.uid);
        n.putAttribute ("gaugeType", gauge.gaugeType);
        n.putAttribute ("gaugeName", gauge.gaugeName);
        n.putAttribute ("modelType", gauge.modelType);
        n.putAttribute ("modelName", gauge.modelName);
        n.putAttribute ("retryCnt", (long)gauge.retryCnt.intValue());
    }

    public static void addGaugeInfo(GaugeID event, Filter f) {
        f.addConstraint ("gaugeType", event.gaugeType);
        f.addConstraint ("gaugeName", event.gaugeName);
        f.addConstraint ("modelType", event.modelType);
        f.addConstraint ("modelName", event.modelName);
    }
        
    public static void extractGaugeID (GaugeID gauge, Notification n) {
        AttributeValue gv = n.getAttribute ("gaugeID");
        if (gv != null) {
            gauge.uid = gv.stringValue ();
        }
        gauge.gaugeType = n.getAttribute ("gaugeType").stringValue ();
        gauge.gaugeName = n.getAttribute ("gaugeName").stringValue ();
        gauge.modelType = n.getAttribute ("modelType").stringValue ();
        gauge.modelName = n.getAttribute ("modelName").stringValue ();
        gauge.retryCnt = new Integer(n.getAttribute ("retryCnt").intValue ());
    }
    
    public static void SPV2Notification (StringPairVector spv, Notification e, String prefix) {
        if (null == e) return;
        
        Integer size = new Integer (spv.size ());
        e.putAttribute ("vectorSize_" + prefix, size.toString ());
        for (int i = 0; i < spv.size (); i++) {
            e.putAttribute ("name_" + prefix + "_" + i, spv.nameAt (i));
            e.putAttribute ("value_" + prefix + "_" + i, spv.valueAt (i));
        }
    }
    
    public static void Notification2SPV (Notification e, StringPairVector spv, String prefix) {
        String pattern = "vectorSize_" + prefix;
        AttributeValue av = e.getAttribute (pattern);
        int size = av.intValue ();
        for (int i = 0; i < size; i++) {
            spv.addElement (e.getAttribute ("name_" + prefix + "_" + i).stringValue (),
                            e.getAttribute ("value_" + prefix + "_" + i).stringValue ());
        }
    }
    
    public static void GVV2Notification (GaugeValueVector gvv, Notification e, String prefix) {
        if (null == e) return;
        
        Integer size = new Integer (gvv.size ());
        e.putAttribute ("vectorSize_" + prefix, size.toString ());
        for (int i = 0; i < gvv.size (); i++) {
            e.putAttribute ("valueName_" + prefix + "_" + i, gvv.nameAt (i));
            e.putAttribute ("propertyName_" + prefix + "_" + i, gvv.propertyAt (i));
            e.putAttribute ("value_" + prefix + "_" + i, gvv.valueAt (i));
        } 
    }
    
    public static void Notification2GVV (Notification e, GaugeValueVector gvv, String prefix) {
        String pattern = "vectorSize_" + prefix;
        int size = e.getAttribute (pattern).intValue ();
        for (int i = 0; i < size; i++) {
            gvv.addElement (e.getAttribute ("valueName_" + prefix + "_" + i).stringValue (),
                            e.getAttribute ("propertyName_" + prefix + "_" + i).stringValue (),
                            e.getAttribute ("value_" + prefix + "_" + i).stringValue ());
        }
    }
    

}
