/*
 * SienaGaugeHandle.java
 *
 * Created on November 2, 2001, 4:44 PM
 * $Header: D:/Projects/Repository/GaugeAPI/edu/cmu/cs/able/gaugeInfrastructure/Siena/SienaGaugeHandle.java,v 1.2 2002/01/31 16:39:06 schmerl Exp $
 * $Log: SienaGaugeHandle.java,v $
 * Revision 1.2  2002/01/31 16:39:06  schmerl
 * Added CVS header information
 *
 */

package edu.cmu.cs.able.gaugeInfrastructure.Siena;
import edu.cmu.cs.able.gaugeInfrastructure.Events.*;
import edu.cmu.cs.able.gaugeInfrastructure.util.*;
import edu.cmu.cs.able.gaugeInfrastructure.*;
import edu.cmu.cs.able.gaugeInfrastructure.Siena.Events.*;
import siena.*;
/**
 * The proxy implementation of a Siena gauge. Objects of this class are held by gauge consumers
 * who interact directly with the gauge through this control interface. Note, however, that
 * gauge consumers get event-based reports by implementing the GaugeConsumer interface and 
 * registering with the gauge bus.
 * @author  Bradley Schmerl
 * @version 1.1
 */
public class SienaGaugeHandle implements edu.cmu.cs.able.gaugeInfrastructure.GaugeControl {

    private SienaGaugeNotifiable sienaHelper;  
    
    private GaugeID gid;
    
    private GaugeConsumer holder;
    
    private boolean consumerInterested = false;
    
    private boolean onlyUID = true;
    
    private boolean interestedInCreated = false;
    
    private boolean interestedInDeleted = false;
    
    private boolean interestedInConfigured = false;
    
    private boolean initialized = false;
    
    /** Creates new SienaGaugeHandle
     * @param gid This ID of the gauge that will be managed by this
     * gauge manager.
     */
    public SienaGaugeHandle(GaugeID gid) {
        sienaHelper = new SienaGaugeNotifiable ();
        this.gid = gid;
        if (gid == null) {
            initialized = true;
        }
    }
    
    /** Returns the ID of the gauge that this is a handle for.
     * @return
     */    
    public GaugeID getGaugeID() {
        return gid;
    }
    
    /** Sets the ID of the gauge handle.
     * @param gaugeID The Gauge ID
     */    
    public void setGaugeID (GaugeID gaugeID) {
        gid = gaugeID;
    }
    
    /** Queries the gauge for all values that the gauge reports.
     * This publishes a request on the Siena bus and waits for a
     * reply from the gauge.
     * @param values Will be filled with a (name, property, value) tuple for
     * each value the gauge reports.
     * @return Whether the values were successfully retrieved from
     * the gauge.
     */    
    public boolean queryAllValues(GaugeValueVector values) {
        QueryAllValuesEvent query = new QueryAllValuesEvent (gid);
 
        // publish request on the gauge bus
         query.values = new GaugeValueVector ();
        query = (QueryAllValuesEvent )Initialization.gaugeConsBus.publishRequest (query);
      
        
        values.removeAllElements ();
        if (query.status) {
         query.values.copyInto (values);
        }
        return query.status;
    }
    
    /** Returns the state of the gauge. Publishes a request on the bus and
     * waits for a reply from the actual gauge.
     * @param setupParams The (name, value) pairs that the gauge was setup with.
     * @param configParams The (name, value) pairs of the configuration of the gauge.
     *
     * @param mappings The (value, property) pairs for each gauge value mapping.
     * @return Whether the information was successfully retrieved from the gauge.
     */    
    public boolean queryState(StringPairVector setupParams, StringPairVector configParams, StringPairVector mappings) {
        QueryGaugeStateEvent query = new QueryGaugeStateEvent (gid);
        
        query = (QueryGaugeStateEvent )Initialization.gaugeConsBus.publishRequest (query);
        //Initialization.gaugeConsBus.waitForReply (query);
        
        setupParams.removeAllElements ();
        configParams.removeAllElements ();
        mappings.removeAllElements ();
        if (query.status) {
            query.configParams.copyInto (configParams);
            query.setupParams.copyInto (setupParams);
            query.mappings.copyInto (mappings);
        }
        return query.status;
    }
    
    public String queryValue(String valueName) {
        QueryValueEvent query = new QueryValueEvent (gid);
        query.valueName = valueName;
        query = (QueryValueEvent )Initialization.gaugeConsBus.publishRequest (query);
        //Initialization.gaugeConsBus.waitForReply (query);
        
        if (query.status) {
            return query.value;
        }
        else {
            return null;
        }
    }
 
    
    public boolean configure(StringPairVector configParams) {
        ConfigureGaugeEvent query = new ConfigureGaugeEvent (gid);
        query.configParams = configParams;
        query = (ConfigureGaugeEvent )Initialization.gaugeConsBus.publishRequest (query);
        ///Initialization.gaugeConsBus.waitForReply (query);
        return query.status;
    }
    

    
    public void registerGaugeConsumer (GaugeConsumer consumer) {
        if (consumer == null) {
            Initialization.gaugeConsBus.unregisterGaugeEvents (holder, this);
//            Initialization.gaugeConsBus.unregisterGaugeEvents (holder, this);
        }
        holder = consumer;
       
        if (consumer != null) {
            Initialization.gaugeConsBus.registerDeletedEvents (holder, this);
            Initialization.gaugeConsBus.registerConfiguredEvents (holder, this);
            Initialization.gaugeConsBus.registerValueEvents (holder,this);
        }
    }
    
}
