/*
 * SienaReportingBus.java
 *
 * Created on November 2, 2001, 7:53 PM
 * $Header: D:/Projects/Repository/GaugeAPI/edu/cmu/cs/able/gaugeInfrastructure/Siena/SienaGaugeReportingBus.java,v 1.3 2002/05/13 14:22:22 zensoul Exp $
 * $Log: SienaGaugeReportingBus.java,v $
 * Revision 1.3  2002/05/13 14:22:22  zensoul
 * Added output of current thread to check for race condition.
 *
 * Revision 1.2  2002/01/31 16:39:07  schmerl
 * Added CVS header information
 *
 */

package edu.cmu.cs.able.gaugeInfrastructure.Siena;
import edu.cmu.cs.able.gaugeInfrastructure.Events.*;
import edu.cmu.cs.able.gaugeInfrastructure.util.*;
import edu.cmu.cs.able.gaugeInfrastructure.*;
import edu.cmu.cs.able.gaugeInfrastructure.Siena.Events.*;
import siena.*;
import java.util.*;
/**
 *
 * @author  schmerl
 * @version 
 */
public class SienaGaugeReportingBus 
        implements edu.cmu.cs.able.gaugeInfrastructure.GaugeReportingBus, Notifiable {

    public static GaugeSideEventBus gaugeSideBus = null;
    public static ConsumerSideEventBus consumerSideBus = null;
    public static SienaGaugeReportingBus reportingBus = null;
    
    private GaugeConsumer consumer = null;
    /** Creates new SienaReportingBus */
    public SienaGaugeReportingBus() {
        gaugeSideBus = Initialization.gaugeBus;
        consumerSideBus = Initialization.gaugeConsBus;
        reportingBus = this;
    }

    
    public boolean reportConfigured(GaugeID gauge, StringPairVector configParams) {
        ConfiguredEvent event = new ConfiguredEvent (gauge);
        event.configParams = configParams;
        gaugeSideBus.publishAnnouncement (event);
        return true;
    }
    
    public boolean reportCreated(GaugeID gauge) {
        CreatedEvent event = new CreatedEvent (gauge);
        gaugeSideBus.publishAnnouncement (event);
        return true;
    }
    
    public boolean reportMultipleValues(GaugeID gauge, GaugeValueVector values) {
        MultipleValueEvent event = new MultipleValueEvent (gauge);
        event.values = values;
        gaugeSideBus.publishAnnouncement (event);
        return true;
    }
    
    public boolean reportValue(GaugeID gauge, String valueName, String propertyName, String value) {
        ValueEvent event = new ValueEvent (gauge);
        event.valueName = valueName;
        event.propertyName = propertyName;
        event.value = value;
        gaugeSideBus.publishAnnouncement (event);
        return true;
    }
    
    public boolean reportDeleted(GaugeID gauge) {
        DeletedEvent event = null;
        if (gauge instanceof DeletedEvent) {
            event = (DeletedEvent )gauge;
        }
        else event = new DeletedEvent (gauge);
        gaugeSideBus.publishAnnouncement (event);
        return true;
    }
    
    

    
    private class Subscription {
        public Subscriber subscriber;
        public Filter filter;
        
        public Subscription (Filter fp, Subscriber sub) {
            filter = fp;
            subscriber = sub;
        }
    }
    
    Vector subscriptions = new Vector ();
    
    private void registerSubscriber (siena.Filter f, Subscriber sub) {
        Subscription scrp = new Subscription (f, sub);
        subscriptions.addElement (scrp);
    }
    

    /** Called when an interested party is wants reports in some pattern.
     * Depending on the underlying implementation, this could allow browsers
     * to register for any creation events, for example.
     * @param event The event pattern to subscribe to.
     * @param subscriber The object that contains a callback to call if the event pattern is matched.
     */    
    public void subscribeToGaugeEvent(GaugeEvent event, Subscriber subscriber) {
        Filter f = EventConverters.event2Filter (event);
        
        try {
            consumerSideBus.siena.subscribe (f, this);
            registerSubscriber (f, subscriber);
        }
        catch (Exception e) {
            e.printStackTrace ();
        }
        
    }

    /** Called to unregister a subscriber's interest in a gauge event pattern.
     * @param event The event pattern to unsubscribe from. If this is <B>null</B>, then the
     * subscriber is unsubscribed from all events.
     * @param subscriber The subscriber to unsubscribe.
     */
    public void unsubscribeToGaugeEvent(GaugeEvent event, Subscriber subscriber) {
        Filter f = EventConverters.event2Filter (event);
        for (int i = 0; i < subscriptions.size (); i++) {
            Subscription scrp = (Subscription )subscriptions.elementAt (i);
            if (Covering.covers (scrp.filter, f) && scrp.subscriber.equals (subscriber)) {
                subscriptions.removeElementAt (i);
                consumerSideBus.siena.unsubscribe (f, this);
                i--;
            }
        }
    }
    
    public void notify (Notification e) {
        try {
        for (int i = 0; i < subscriptions.size (); i++) {
            Subscription sub = (Subscription )subscriptions.elementAt (i);
            if (siena.Covering.apply (sub.filter, e)) {
                sub.subscriber.receiveEvent (EventConverters.notification2Event (e));
            }
        }
        }
        catch (Exception ex) {
            ex.printStackTrace ();
        }
    }
    
    public void notify (Notification [] e) {
        for (int i = 0; i < e.length; i++) {
            notify (e [i]);
        }
    }
    
    public static GaugeReportingBus getReportingBus() {
        return reportingBus;
 
    }
    
/*    public void registerGaugeConsumer(GaugeConsumer consumer, GaugeControl gauge) {
        if (!(sienaBus.registerDeletedEvents (consumer, gauge) &&
              sienaBus.registerCreatedEvents (consumer, gauge) &&
              sienaBus.registerConfiguredEvents (consumer, gauge) &&
              sienaBus.registerValueEvents (consumer, gauge))) {
            System.err.println ("Could not register for events");         
        }
    }
 */
    
    public void registerGauge (GaugeControl gauge, StringPairVector setupParams, 
                               StringPairVector mappings, boolean waitForInit, String myManagerID) {
 
        gaugeSideBus.registerGauge (gauge); 
        if (waitForInit) {
            Global.debug ("Sending for initialization");
            Global.debug (" -> SienaGaugeReportingBus current thread is " + Thread.currentThread());
            GaugeInitEvent request = new GaugeInitEvent (gauge.getGaugeID ());
            request.gaugeMgrID = myManagerID;
            request = (GaugeInitEvent )gaugeSideBus.publishRequest (request);
            //sienaBus.waitForReply (request);
            if (!request.status) {
                Global.debug ("Could not get initialization parameters");
            }
            else {
                Global.debug ("Got initialization");
                request.mappings.copyInto (mappings);
                request.setup.copyInto (setupParams);
            }
            
            ((SienaGauge )gauge).setGaugeID (request); 
        }

    }
    


}
